
""" This class contains all the functions for storing stuff in the database """

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# Copyright 2004 Dries Verachtert

# if you want to have a pydar which works without a database or with another type of database
# then you have to write a new subclass of Storage

import sys
sys.path.append("pydar")
sys.path.append(".")
from mastercommand import MasterCommand
from log4py import Logger
import config
from storage import Storage
import pgdb, posixpath, posix, string
import rights

class PostgresqlStorage(Storage):
    def __init__(self):
        self.__cat = Logger().get_instance(self)
        self.__cat.debug("initialized")

    def isBuildMachineId(self, id):
        retval = False
        conn = self.__getConnection()
        cursor = conn.cursor()
        sql = "select id from pydar2_distroarch where id='" + id + "'"
        cursor.execute(sql)
        for row in cursor.fetchall():
            retval = True
        cursor.close()
        self.__releaseConnection(conn)
        return retval
        
    def getAuthorityOfMostRecentVersion(self, specFileId):
        retval = None
        conn = self.__getConnection()
        cursor = conn.cursor()
        sql = "select t.value from pydar2_specfile_version sv, pydar2_specfile_tags t where t.versionid=sv.id and upper(t.name)='# AUTHORITY' and sv.specfileid=" + str(specFileId)  + " order by sv.id desc limit 1"
        self.__cat.debug("sql:" + sql)
        cursor.execute(sql)
        for row in cursor.fetchall():
            retval = row[0]
        cursor.close()
        self.__releaseConnection(conn)
        return retval
        
    def getNameOfMostRecentVersion(self, specFileId):
        retval = None
        conn = self.__getConnection()
        cursor = conn.cursor()
        sql = "select t.value from pydar2_specfile_version sv, pydar2_specfile_tags t where t.versionid=sv.id and upper(t.name)='NAME' and sv.specfileid=" + str(specFileId)  + " order by sv.id desc limit 1"
        self.__cat.debug("sql:" + sql)
        cursor.execute(sql)
        for row in cursor.fetchall():
            retval = row[0]
        cursor.close()
        self.__releaseConnection(conn)
        return retval
        
    def getSpecFileIdsToVersions(self, specRepository):
        retval = {}
        conn = self.__getConnection()
        cursor = conn.cursor()
        sql = "select sv.specfileid, max(version) as maxversion from pydar2_specfile_version sv, pydar2_specfile v where sv.specfileid=v.id and v.specrepoid=" + str(specRepository.getId()) + " group by sv.specfileid order by maxversion desc"
        cursor.execute(sql)
        for row in cursor.fetchall():
            retval[row[0]] = row[1]
        cursor.close()
        self.__releaseConnection(conn)
        return retval
    
    def getTargetRpmsSpecFileIdsToVersions(self,targetObj, datag,specRepository):
        retval = {}
        conn = self.__getConnection()
        cursor = conn.cursor()
        sql = "select specfileid, max(version) from (select sv.specfileid, sv.version from pydar2_specfile_version sv, pydar2_specfile_tags t1, pydar2_target_files tf, pydar2_specfile s where tf.name=t1.value and t1.name='NAME' and t1.versionid=sv.id and s.id=sv.specfileid and s.specrepoid=" +  str(specRepository.getId())+ " and tf.targetid=" +  str(targetObj.getId() )+ " and tf.distroarchtag='" + datag + "') as foo1 group by specfileid"
        cursor.execute(sql)
        for row in cursor.fetchall():
            retval[row[0]] = row[1]
        cursor.close()
        self.__releaseConnection(conn)
        return retval
    
        
    def updateTargetFile(self, targetObj, fileName, checksum, version, distroarchtag, name):
        self.__cat.debug("start, fileName:"  + str(fileName) + ",checksum:" + str(checksum) + ",version:" + str(version) + ",distroarchtag:" + str(distroarchtag) + ",name:" + str(name))
        conn = self.__getConnection()
        cursor = conn.cursor()
        sql = "update pydar2_target_files set targetid=" + str(targetObj.getId()) + ", version=" + str(version )+ ",name='"+ str(name) + "',checksum='"+ str(checksum) + "',distroarchtag='"+distroarchtag +"' where fileName='" + fileName + "'"
        cursor.execute(sql)
        conn.commit()
        cursor.close()
        self.__releaseConnection(conn)
    
    def insertTargetFile(self, targetObj, fileName, checksum, version, distroarchtag, name):
        self.__cat.debug("start, fileName:"  + str(fileName) + ",checksum:" + str(checksum) + ",version:" + str(version) + ",distroarchtag:" + str(distroarchtag) + ",name:" + str(name))
        conn = self.__getConnection()
        cursor = conn.cursor()
        sql = "insert into pydar2_target_files (targetid, filename, name, version, checksum, distroarchtag) values (" + str(targetObj.getId()) + ",'"+ fileName + "','"+ name + "'," + str(version) + ",'" + str(checksum) + "','"+ distroarchtag+ "')"
        cursor.execute(sql)
        conn.commit()
        cursor.close()
        self.__releaseConnection(conn)
    
    def getDistroArchTags(self):
        retval = []
        conn = self.__getConnection()
        cursor = conn.cursor()
        sql = "select distroarchtag from pydar2_distroarch"
        cursor.execute(sql)
        for row in cursor.fetchall():
            retval.append(row[0])
        cursor.close()
        self.__releaseConnection(conn)
        return retval
    
    def getSpecRepositoryFileListToIds(self,specRepo):
        retval = {}
        conn = self.__getConnection()
        cursor = conn.cursor()
        sql = "select subdir, filename, id from pydar2_specfile where specrepoid=" + str(specRepo.getId())
        cursor.execute(sql)
        if cursor.rowcount > 0:
            arr = cursor.fetchall()
            for row in arr:
                subdir = row[0]
                filename = row[1]
                id = row[2]
                retval[subdir + '/' + filename] = id
        cursor.close()
        self.__releaseConnection(conn)
        return retval
        
    # this gets the newest version of a spec file. This is not the same as the id of the newest version
    # the version might be for example the svnlatestcommitrevision of a file and is unique for 1 spec file
    # the version id is unique for all spec files for all spec repositories
    def getNewestVersion(self, specId):
        self.__cat.debug("start")
        retval = None
        conn = self.__getConnection()
        cursor = conn.cursor()
        sql = "select version from pydar2_specfile_version where specfileid=" + str(specId)
        cursor.execute(sql)
        if cursor.rowcount > 0:
            row= cursor.fetchone()
            retval = row[0]
        cursor.close()
        self.__releaseConnection(conn)
        return retval
        
        
    def saveSpecRepositorySpecFile(self, specRepo, specRepoSpecFile):
        conn = self.__getConnection()
        cursor = conn.cursor()
        if specRepoSpecFile.getId() != None and specRepoSpecFile.getId() >= 0:
            # it should be in the database..
            sql = "update pydar2_specfile set filename='" + str(specRepoSpecFile.getFileName()) + "', "
            sql = sql + " subdir='" + str(specRepoSpecFile.getSubDir()) + "', "
            sql = sql + " specrepoid=" + str(specRepo.getId()) + ", "
            sql = sql + " checksum='" + str(specRepoSpecFile.getCheckSum()) + "' "
            sql = sql + " where id=" + str(specRepoSpecFile.getId())
            cursor.execute(sql)
            conn.commit()
        else:
            # new spec file, insert it
            sql = "insert into pydar2_specfile (specrepoid, filename, subdir, checksum) values ("
            sql = sql + "" + str(specRepo.getId()) + ","
            sql = sql + "'" + str(specRepoSpecFile.getFileName()) + "',"
            sql = sql + "'" + str(specRepoSpecFile.getSubDir()) + "',"
            sql = sql + "'" + str(specRepoSpecFile.getCheckSum()) + "')"
            cursor.execute(sql)
            conn.commit()
        # now get the id so we can return it at the end
        cursor.execute("select id from pydar2_specfile where filename='" + str(specRepoSpecFile.getFileName()) + "' and subdir='" + str(specRepoSpecFile.getSubDir()) + "' and specrepoid=" + str(specRepo.getId()))
        retval = None
        if cursor.rowcount > 0:
            arr = cursor.fetchall()
            for row in arr:
                retval = row[0]
        cursor.close()
        self.__releaseConnection(conn)
        specRepoSpecFile.setId(retval)
        # now check if we also have to save all tags of this spec file
        if self.getConfig().getSaveSpecRepositorySpecFileTags():
            self.__saveSpecRepositorySpecFileTags(specRepo,specRepoSpecFile)
        return retval
    
    def __isSpecRepositorySpecFileVersionInfoPresent(self,specfileid,currentversion):
        conn = self.__getConnection()
        cursor = conn.cursor()
        sql = "select version from pydar2_specfile_version where specfileid=" + str(specfileid) + " and version=" + str(currentversion)
        self.__cat.debug(sql)
        cursor.execute(sql)
        retval = 0
        if cursor.rowcount > 0: 
            retval = 1
        cursor.close()
        self.__releaseConnection(conn)
        return retval
    
    def __insertSpecRepositorySpecfileVersion(self,specfileid,currentversion):
        conn = self.__getConnection()
        cursor = conn.cursor()
        cursor.execute("insert into pydar2_specfile_version (specfileid,version) values (" + str(specfileid) + "," + str(currentversion) + ")")
        conn.commit()
        cursor.close()
        self.__releaseConnection(conn)
    
    def __saveSpecRepositorySpecFileTags(self,specRepo,specRepoSpecFile):
        self.__cat.debug("start")
        specfileid = specRepoSpecFile.getId()
        currentversion = specRepoSpecFile.getVersion()
        self.__cat.debug("current version is " + str(currentversion))
        alreadyImported = self.__isSpecRepositorySpecFileVersionInfoPresent(specfileid,currentversion)
        if alreadyImported:
            # strange.. this function is normally only called when the file is changed
            # but the cvs or svn or .. version is not changed => somebody changed the file in the privat e
            self.__cat.warn("already imported in db, specRepo=" + specRepo.getName() + ",file id=" + str(specRepoSpecFile.getId()))
        else:
            self.__cat.debug("need to import all the variables..")
            self.__insertSpecRepositorySpecfileVersion(specfileid,currentversion)
            self.__insertSpecRepositorySpecFileTagsPerDistro(specRepo, specRepoSpecFile, currentversion)
    
    # returns a hash wich contains the fields    code => defines  from the db table  pydar2_distro
    def __getDistroCodeToDefine(self):
        retval = {}
        conn = self.__getConnection()
        cursor = conn.cursor()
        sql = "select distroarchtag, defines from pydar2_distroarch where disabled='0'"
        cursor.execute(sql)
        rsdistros = cursor.fetchall()
        for row in rsdistros:
            code = row[0]
            defines = row[1]
            print "code=" + code
            print "defines=" + defines
            retval[code] = defines
        cursor.close()
        self.__releaseConnection(conn)
        return retval
    
    def __insertSpecRepositorySpecFileTagsPerDistro(self,specRepo, specRepoSpecFile,currentversion):
        specfileid= specRepoSpecFile.getId()
        self.__cat.debug("start,specfileid=" + str(specRepoSpecFile.getId()) + ",currentversion=" + str(currentversion))
        distros = self.__getDistroCodeToDefine()
        conn = self.__getConnection()
        cursor = conn.cursor()
        # first get the versionid
        sql = "select id from pydar2_specfile_version where specfileid=" + str(specRepoSpecFile.getId()) + " and version=" + str(currentversion)
        versionid = None
        cursor.execute(sql)
        rstmp = cursor.fetchall()
        for row in rstmp:
            versionid = row[0]
        self.__cat.debug("versionid=" + str(versionid))
        for distrocode in distros.keys():
            defines = distros[distrocode]
            sf = specRepoSpecFile.getSpecFileTags(defines)
            
            for tag in sf.rpmforgetags:
                print 'rpmforge tag: ' + tag
                name = tag
                #print 'value: ' + sf.rpmforgetags[tag]
                value = string.replace(sf.rpmforgetags[tag],"'","''")
                value = string.rstrip(value,"\\")
                cursor.execute("insert into pydar2_specfile_tags (versionid,distroid,name,value) values (" + str(versionid) + ",(select id from pydar2_distroarch where distroarchtag='" + str(distrocode) + "'),'"+ str(name) + "','" + str(value) + "')")
            for tag in sf.standardtags:
                print 'standard tag: ' + tag
                name = tag
                # print 'value: ' + string.replace(sf.standardtags[tag],"'","''")
                value = string.replace(sf.standardtags[tag],"'","''")
                value = string.rstrip(value,"\\")
                cursor.execute("insert into pydar2_specfile_tags (versionid,distroid,name,value) values (" + str(versionid)  + ",(select id from pydar2_distroarch where distroarchtag='" + str(distrocode) + "'),'"+ str(name) + "','" + str(value) + "')")
        
        conn.commit()
        cursor.close()
        self.__releaseConnection(conn)
    
    def saveSpecRepository(self,specRepo):
        conn = self.__getConnection()
        cursor = conn.cursor()
        cursor.execute("select * from pydar2_specrepo where name='" + str(specRepo.getName()) + "'")
        if cursor.rowcount > 0:
            # specrepo exists, update it
            sql = "update pydar2_specrepo set rootdirectory='" + str(specRepo.getRootDirectory()) + "', type='"
            sql = sql + str(specRepo.getType()) + "', deleted='0' where name='" + str(specRepo.getName()) + "'"
            cursor.execute(sql)
            conn.commit()
        else:
            # specrepo does not exist, add it
            sql = "insert into pydar2_specrepo (name, rootdirectory, type) values ('" + str(specRepo.getName()) + "','" + str(specRepo.getRootDirectory()) + "','" + str(specRepo.getType()) + "')"
            cursor.execute(sql)
            conn.commit()
        # now get the id again
        cursor.execute("select id from pydar2_specrepo where name='" + str(specRepo.getName()) + "'")
        retval = None
        arr = cursor.fetchall()
        for row in arr:
            retval = row[0]
        cursor.close()
        self.__releaseConnection(conn)
        return retval
        
    # @todo add some type of connection pooling 
    def __getConnection(self):
        conn = pgdb.connect(config.Config.getInstance().getPostgresqlConnectString())
        return conn
        
    def __releaseConnection(self,conn):
        conn.close()
    
    def getTargetFileListToChecksums(self, targetObj):
        self.__cat.debug("start")
        retval = {}
        conn = self.__getConnection()
        cursor = conn.cursor()
        sql = "select filename, checksum from pydar2_target_files where targetid=" + str(targetObj.getId())
        cursor.execute(sql)
        if cursor.rowcount > 0:
            arr = cursor.fetchall()
            for row in arr:
                filename = row[0]
                checksum = row[1]
                retval[ filename] = checksum
        cursor.close()
        self.__releaseConnection(conn)
        return retval
        
    
    # returns a hash with ( __subDir + '/' + __fileName ) => ( checksum )
    def getSpecRepositoryFileListToChecksum(self,specRepo):
        retval = {}
        conn = self.__getConnection()
        cursor = conn.cursor()
        sql = "select subdir, filename, checksum from pydar2_specfile where specrepoid "
        sql = sql + "in (select id from pydar2_specrepo where name='" + specRepo.getName() + "')"
        cursor.execute(sql)
        if cursor.rowcount > 0:
            arr = cursor.fetchall()
            for row in arr:
                subdir = row[0]
                filename = row[1]
                checksum = row[2]
                retval[subdir + '/' + filename] = checksum
        cursor.close()
        self.__releaseConnection(conn)
        return retval
        
    # returns the command, but with the id set
    def createCommand(self,aCommand):
        self.__cat.debug("start")
        conn = self.__getConnection()
        cursor = conn.cursor()
        cursor.execute("select nextval('pydar2_commands_id_seq')")
        id = -1
        temparr = cursor.fetchall()
        for row in temparr:
            id = row[0]
        #commandName,userId,specrepoid,specfileid,toEmail,distroArchTag,priority,targetid
        cursor.execute("insert into pydar2_commands (id,commandname,userid,specrepoid,specfileid,toemail,distroarchtag,priority,targetid,version) values ("
        + str(id) + ",'"
        + aCommand.getCommandName() + "','"
        + str(aCommand.getUserId()) + "',"
        + str(aCommand.getSpecRepoId()) + ","
        + str(aCommand.getSpecFileId()) + ",'"
        + str(aCommand.getToEmail()) + "','"
        + str(aCommand.getDistroArchTag()) + "',"
        + str(aCommand.getPriority()) + ","
        + str(aCommand.getTargetId()) + ","
        + "(select max(version) from pydar2_specfile_version sv where sv.specfileid="+str(aCommand.getSpecFileId())+"))")
        conn.commit()
        cursor.close()
        self.__releaseConnection(conn)
        aCommand.setId(id)
        return aCommand

    def checkIfValidUserId(self, userId, password, right):
        self.__cat.debug("start, userId=" + str(userId))
        retval = 0
        conn = self.__getConnection()
        cursor = conn.cursor()
        cursor.execute("select * from pydar2_users u, pydar2_rights r, pydar2_users_to_rights ur where u.id=ur.userid and ur.rightid=r.id and r.abbrev='"+ str(right)  + "' and u.id='" + str(userId) + "' and u.password='" + str(password) + "'")
        if cursor.rowcount > 0:
            retval = 1
        cursor.close()
        self.__releaseConnection(conn)
        self.__cat.debug("retval=" + str(retval))
        return retval

    def checkIfValidBuildMachineId(self, buildmachineid, password, right):
        self.__cat.debug("start, id=" + buildmachineid)
        conn = self.__getConnection()
        retval = 0
        cursor = conn.cursor()
        cursor.execute("select * from pydar2_buildmachines u, pydar2_rights r, pydar2_buildmachines_to_rights ur where u.id=ur.buildmachineid and ur.rightid=r.id and r.abbrev='"+ str(right)  + "' and u.id='" + str(buildmachineid) + "' and u.password='"+ str(password)  + "'")
        if cursor.rowcount > 0:
            retval = 1
        cursor.close()
        self.__releaseConnection(conn)
        self.__cat.debug("retval=" + str(retval))
        return retval

    def updateTarget(self, newTarget):
        self.__cat.debug("start")
        # check if exits
        conn = self.__getConnection()
        cursor = conn.cursor()
        sql = "select * from pydar2_targets where name='" + newTarget.getName() + "'"
        cursor.execute(sql)
        if cursor.rowcount == 0:
            sql = "insert into pydar2_targets (name) values ('" + newTarget.getName() + "')"
            cursor.execute(sql)
            conn.commit()
        # else:
            # nothing to update yet..
        sql = "select id from pydar2_targets where name='" + newTarget.getName() + "'"
        cursor.execute(sql)
        newId = None
        if cursor.rowcount > 0:
            rs = cursor.fetchone()
            newId = rs[0]
            newTarget.setId(newId)
        cursor.close()
        self.__releaseConnection(conn)
        
    def registerSlave(self, buildmachineid):
        self.__cat.debug("start, buildmachineid=" + buildmachineid)
        conn = self.__getConnection()
        cursor = conn.cursor()
        cursor.execute("insert into pydar2_buildmachine_registrations (id) values ('" + buildmachineid + "')")
        cursor.execute("delete from pydar2_buildmachine_distroarchtags where buildmachineid='" + buildmachineid + "'")
        conn.commit()
        cursor.close()
        self.__releaseConnection(conn)

    def addDistroArchTag(self, buildmachineid, distroArchTag):
        self.__cat.debug("start, id=" + buildmachineid + ",distroArchTag=" + distroArchTag)
        conn = self.__getConnection()
        cursor = conn.cursor()
        cursor.execute("insert into pydar2_buildmachine_distroarchtags (buildmachineid,distroarchtag) values ('" + buildmachineid + "','" + distroArchTag + "')")
        conn.commit()
        cursor.close()
        self.__releaseConnection(conn)

    def addResultFileName(self,buildmachineid,commandId,fileName):
        # no check on buildmachineid yet...
        conn = self.__getConnection()
        cursor = conn.cursor()
        cursor.execute("insert into pydar2_actions_files (commandid,filename) values (" + str(commandId) + ",'" + fileName + "')")
        conn.commit()
        cursor.close()
        self.__releaseConnection(conn)

    def setBuildResult(self,buildmachineid,commandId,buildResult):
        conn = self.__getConnection()
        cursor = conn.cursor()
        cursor.execute("insert into pydar2_actions (buildmachineid,commandid,statusid) values ('" + buildmachineid + "'," + str(commandId) + "," + str(buildResult+1) + ")")
        cursor.execute("update pydar2_commands set handled=true,inprogress=false where id=" + str(commandId))
        conn.commit()
        cursor.close()
        self.__releaseConnection(conn)

    def getNumberOfNotYetStartedCommands(self):
        retval = None
        conn = self.__getConnection()
        cursor = conn.cursor()
        cursor.execute("select count(*) as total from pydar2_commands where not pc.handled and not pc.inprogress")
        if cursor.rowcount > 0:
            row = cursor.fetchone()
            retval = row[0]
        conn.commit()
        cursor.close()
        self.__releaseConnection(conn)
        return retval
        
    def reserveCommand(self, buildmachineid):
        self.__cat.debug("start, buildmachineid=" + buildmachineid)
        conn = self.__getConnection()
        cursor = conn.cursor()
        sql = "select pc.id, pc.specfileid, pc.specrepoid, pc.toemail, pc.distroarchtag, pc.targetid, pc.commandname, pc.userid, pc.priority, pc.version from pydar2_commands pc where pc.commandname='BUILDSPECBYPATH' and not pc.handled and not pc.inprogress and pc.distroarchtag in (select distroarchtag from pydar2_buildmachine_distroarchtags where buildmachineid='" + buildmachineid + "')  order by pc.priority desc, pc.distroarchtag desc, pc.id asc limit 1"
        self.__cat.debug("sql 1: " + sql)
        cursor.execute(sql)
        retval = ""
        if cursor.rowcount > 0:
            rs = cursor.fetchone()
            self.__cat.debug("command found for buildmachineid=" + buildmachineid + ",rs=" + str(rs))
            commandId = rs[0]
            specFileId = rs[1]
            specRepoId = rs[2]
            toEmail = rs[3]
            distroArchTag = rs[4]
            targetId = rs[5]
            commandName = rs[6]
            userId = rs[7]
            priority = rs[8]
            version = rs[9]
            aCommand = MasterCommand(commandName,userId,specRepoId,specFileId,toEmail,distroArchTag,priority,targetId)
            aCommand.setVersion(version)
            self.__cat.debug("command id: " + str(rs[0]))
            aCommand.setCommandId(rs[0])
            self.__cat.debug("id according to command object: " + str(aCommand.getId()))
            cursor.execute("update pydar2_commands set inprogress=true where id=" + str(aCommand.getId()))
            cursor.execute("insert into pydar2_actions (commandid, statusid, buildmachineid) values (" + str(aCommand.getId()) + ",0,'" + str(buildmachineid) + "')")
            retval = aCommand
        else:
            self.__cat.debug("nothing to do for buildmachineid=" + buildmachineid)
            retval = ""
        conn.commit()
        cursor.close()
        self.__releaseConnection(conn)
        return retval

    def getCommand(self, commandId):
        self.__cat.debug("start, commandId=" + commandId)
        conn = self.__getConnection()
        cursor = conn.cursor()
        sql = "select pc.id, pc.specfileid, pc.specrepoid, pc.toemail, pc.distroarchtag, pc.targetid, pc.commandname, pc.userid, pc.priority, pc.version from pydar2_commands pc where pc.id=" + str(commandId)
        self.__cat.debug("sql 1: " + sql)
        cursor.execute(sql)
        retval = ""
        if cursor.rowcount > 0:
            self.__cat.debug("rowcount > 0")
            rs = cursor.fetchone()
            self.__cat.debug("command found for commandId=" + str(commandId))
            commandId = rs[0]
            specFileId = rs[1]
            specRepoId = rs[2]
            toEmail = rs[3]
            distroArchTag = rs[4]
            targetId = rs[5]
            commandName = rs[6]
            userId = rs[7]
            priority = rs[8]
            version = rs[9]
            aCommand = MasterCommand(commandName,userId,specRepoId,specFileId,toEmail,distroArchTag,priority,targetId)
            aCommand.setVersion(version)
            aCommand.setCommandId(rs[0])
            retval = aCommand
        else:
            self.__cat.debug("rowcount <= 0")
            retval = None
        conn.commit()
        cursor.close()
        self.__releaseConnection(conn)
        return retval
        
    def getBuildMachineIdOfCommand(self, commandId):
        retval = None
        conn = self.__getConnection()
        cursor = conn.cursor()
        sql = "select buildmachineid, count(buildmachineid) as cnt from pydar2_actions where commandid=" + str(commandId) + " group by buildmachineid order by cnt desc"
        cursor.execute(sql)
        if cursor.rowcount > 0:
            row = cursor.fetchone()
            retval = row[0]
        cursor.close()
        self.__releaseConnection(conn)
        return retval
    
    def getVersionIdsOfSpecRepoSpecFile(self, specRepoSpecFile):
        conn = self.__getConnection()
        cursor = conn.cursor()
        retval = []
        sql = "select id from pydar2_specfile_version where specfileid=" + str(specRepoSpecFile.getId()) + " order by id desc"
        cursor.execute(sql)
        all = cursor.fetchall()
        for row in all:
            retval.append(row[0])
        cursor.close()
        self.__releaseConnection(conn)
        return retval
        
    # returns an array of hashes with keys name, value, distroarchtag
    def getTagsOfSpecFileVersion(self, versionId):
        conn = self.__getConnection()
        cursor = conn.cursor()
        retval = []
        sql = "select t1.name, t1.value, d.distroarchtag from pydar2_specfile_tags t1, pydar2_distroarch d where t1.distroid=d.id and  versionid=" + str(versionId) + " order by t1.name asc, t1.distroid asc"
        cursor.execute(sql)
        for row in cursor.fetchall():
            h = {}
            h["name"] = row[0]
            h["value"] = row[1]
            h["distroarchtag"] = row[2]
            retval.append(h)
        cursor.close()
        self.__releaseConnection(conn)
        return retval
        
    # get a mapping from something like fc3-i386 to --define fc3 1 --define dist fc3 for example
    def getDefinesByDistroArchTag(self,distroArchTag):
        conn = self.__getConnection()
        cursor = conn.cursor()
        sql = "select defines from pydar2_distroarch where distroarchtag='" + distroArchTag + "'"
        retval = ""
        cursor.execute(sql)
        if cursor.rowcount > 0:
            rs = cursor.fetchone()
            retval = rs[0]
        cursor.close()
        self.__releaseConnection(conn)
        return retval
    
    def clearAssignedRights(self):
        conn = self.__getConnection()
        cursor = conn.cursor()
        sql = "delete from pydar2_users_to_rights"
        cursor.execute(sql)
        sql = "delete from pydar2_buildmachines_to_rights"
        cursor.execute(sql)
        conn.commit()
        cursor.close()
        self.__releaseConnection(conn)
        
    def addAccount(self,name,type,fullname,passw,rights):
        conn = self.__getConnection()
        cursor = conn.cursor()
        # does the user already exist?
        tablename = "pydar2_users"
        if type=="BUILDMACHINE":
            tablename="pydar2_buildmachines"
        sql = "select * from " + tablename + " where id='" + name + "'"
        cursor.execute(sql)
        if cursor.rowcount > 0:
            # update
            sql = "update " + tablename + " set name='" + fullname + "', password='" +str(passw)+ "' where id='" + name + "'"
        else:
            sql = "insert into " + tablename + " (id,name,password) values ('" + str(name) + "','" + str(fullname) + "','" + str(passw) + "')"
        cursor.execute(sql)
        # add the rights
        tablename = "pydar2_users_to_rights"
        firstfield = "userid"
        if type=="BUILDMACHINE":
            tablename = "pydar2_buildmachines_to_rights"
            firstfield="buildmachineid"
        for r in rights:
            sql = "insert into " + str(tablename) + " (" + str(firstfield) + ",rightid) values ('" + str(name) + "',(select id from pydar2_rights where abbrev='" + str(r) + "'))"
            cursor.execute(sql)
        conn.commit()
        cursor.close()
        self.__releaseConnection(conn)
