
""" Update info about rpm files (from the apt/yum repository) in a db """

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# Copyright 2004 Dries Verachtert


# Normally this runs on the master server. It checks all the rpms in a 
# repository
# Probably implemented as:
# every x hours a cronjob starts which tries to update a mirror of the repository and
# checks which files are changed. The information about those rpms is updated in the
# database.

import re, os, commands, posix, posixpath, string, pgdb
from log4py import Logger
from svnwrapper import SvnWrapper
from rpmfile import RpmFile

class RpmsCheck:
    def __init__(self):
        self.cat = Logger().get_instance(self)
        self.cat.debug("start")
        
    # all the rpms in the rpmsDir will be read, not in subdirectories
    # so call this function for every directory which contains rpms
    def check(self, rpmsDir,dbconnectstring):
        self.cat.debug("start, rpmsDir=" + rpmsDir)
        self.dbconnectstring = dbconnectstring
        self.cat.debug("self.dbconnectstring set to: " + self.dbconnectstring)
        self.dbconnection = pgdb.connect(self.dbconnectstring)
        locationId = self.getRpmLocationId(rpmsDir)
        # let's check every spec file we can find in the repository
        rpmsDirList = posix.listdir(rpmsDir)
        for file in rpmsDirList:
            rindex = string.rfind(file,'.rpm')
            if rindex > 0 and string.rfind(file,'-devel-') < 0 and string.rfind(file,'-debuginfo-') < 0 and rindex == len(file) - len('.rpm'):
                self.cat.debug("rpm match, file=" + file)
                self.checkRpmFile(rpmsDir,file,locationId);

    def getRpmLocationId(self,rpmsDir):
        # inserts a new one if needed!
        cursor = self.dbconnection.cursor()
        cursor.execute("select id from pydar2_rpmlocation where path='" + rpmsDir + "'")
        if cursor.rowcount > 0:
            rs = cursor.fetchone()
            retval = rs[0]
            cursor.close()
            return str(retval)
        else:
            cursor.execute("insert into pydar2_rpmlocation (path) values ('" + rpmsDir + "')")
            self.dbconnection.commit()
            cursor.close()
            return self.getRpmLocationId(rpmsDir)
                
    def checkRpmFile(self,rpmsDir,fileName,locationId):
        self.cat.debug("start,rpmsDir="+rpmsDir+",fileName="+fileName+",locationId=" + locationId)
        rf = RpmFile(rpmsDir+"/"+fileName,'')
        # locationid : locationId
        # rpmname : fileName
        # path : rpmsDir
        specRpmName = rf.standardtags['NAME']
        # specrpmname : specRpmName
        specRpmVersion = rf.standardtags['VERSION']
        specRpmEpoch = rf.standardtags['EPOCH'];
        specRpmRelease = rf.standardtags['RELEASE']
        # specrpmrelease : specRpmRelease
        fileMd5s = rf.standardtags['FILEMD5S']
        # filemd5s : fileMd5s
        chlogLine = rf.standardtags['CHANGELOGNAME']
        regexp = '.*#([0-9]*)$'
        regex = re.compile(regexp)
        result = regex.match(chlogLine)
        svnCmtRev = "null";
        specFileId = "null";
        if result:
            svnCmtRev = result.group(1)
            self.cat.debug("svnCmtRev found: " + svnCmtRev)
            specFileId = self.getSpecFileId(svnCmtRev,specRpmName,specRpmRelease)
        # svncmtrev : svnCmtRev
        # specfileid : specFileId
        self.checkRpmFileInDb(locationId,fileName,rpmsDir,specRpmName,specRpmRelease,fileMd5s,svnCmtRev,specFileId,specRpmVersion,specRpmEpoch)
        
    def checkRpmFileInDb(self,locationId,fileName,rpmsDir,specRpmName,specRpmRelease,fileMd5s,svnCmtRev,specFileId,specRpmVersion,specRpmEpoch):
        self.cat.debug('start,fileName=' + fileName + ",svnCmtRev=" + svnCmtRev + ",specFileId=" + specFileId)
        sSqlStmt = "select * from pydar2_rpmfile where rpmname='" + fileName + "' and specrpmversion='" + specRpmVersion + "' and specrpmepoch='" + specRpmEpoch + "' and locationid=" + locationId + " and specrpmname='" + specRpmName + "' and specrpmrelease='" + specRpmRelease + "' and filemd5s='" + fileMd5s + "' and  svnCmtRev=" + svnCmtRev + " and specfileid=" + specFileId;
        cursor = self.dbconnection.cursor()
        cursor.execute(sSqlStmt)
        if cursor.rowcount == 0:
                cursor.execute("insert into pydar2_rpmfile (locationid,rpmname,path,specrpmname,specrpmrelease,filemd5s,svncmtrev,specfileid,specrpmversion,specrpmepoch) values (" + locationId + ",'" + fileName + "','" + rpmsDir + "','" + specRpmName + "','" + specRpmRelease + "','" + fileMd5s + "'," + svnCmtRev + "," + specFileId + ",'" + specRpmVersion + "','" + specRpmEpoch + "')")
                self.dbconnection.commit()
                cursor.close()
                
        
    def getSpecFileId(self,svnCmtRev,specRpmName,specRpmRelease):
        sqlStr = "select v.specfileid from pydar2_specfile_version v, pydar2_specfile_tags t1 where t1.name = 'NAME' and t1.specfileid=v.specfileid and t1.svncmtrev=v.svncmtrev and v.svncmtrev=" + svnCmtRev + " and '" + specRpmName + "' like t1.value || '%' order by t1.value desc"
        self.cat.debug('sqlStr: ' + sqlStr)
        cursor = self.dbconnection.cursor()
        cursor.execute(sqlStr)
        if cursor.rowcount > 0:
            rs = cursor.fetchone()
            retval = rs[0]
            cursor.close()
            self.cat.debug('retval:' + str(retval))
            return str(retval)
        else:
            retval = "null"
            cursor.close()
            return retval
