
""" A buildroot based on smart """

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# Copyright 2004 Dries Verachtert


import getopt, os, sys, Queue, posix, posixpath, string, re, os.path, shutil, commands
from log4py import Logger
from buildroot import BuildRoot
from buildrootresult import BuildRootResult
import config
#from specfile import SpecFile

# import stuff:
# -odata-dir=/var/lib/pydar2/smart/ <buildmachineid> / <buildroottag> /
# -orpm-root=/var/lib/pydar2/roots/ <buildmachineid> / <buildroottag>-smart /
#  todo: how to change the dir /etc/smart/channels ?

# not finished, todo

class SmartBasedBuildRoot(BuildRoot):
    def __init__(self,buildRootTag):
        self.__cat = Logger().get_instance(self)
        brr = BuildRootResult(self)
        self.__config = config.Config.getInstance()
        self._setBuildRootTag( buildRootTag)
        self.__setVars(brr)
        self.__cat.debug("initialized")
        self.__useCacheOnly = False
        #return brr
        
    # todo possible with smart?
    def setUseCacheOnly(self,bVar):
        self.__useCacheOnly = bVar
    
    def _umounts(self, brr):
        self.__cat.debug("start")
        cmd = "sudo umount " + self.rootdir + "/proc"
        self.logCommandWithoutErrors(cmd,brr)
        cmd = "sudo umount " + self.rootdir + "/sys"
        self.logCommandWithoutErrors(cmd,brr)
        
    def cleanRoot(self):
        self.__cat.debug("cleaning this root..")
        brr = BuildRootResult(self)
        self._umounts(brr)
        cmd = "sudo rm -Rf " + self.rootdir
        self.logCommand(cmd,brr)
        return brr
        
    def __setVars(self,brr):
        self.yumConfFile = "/etc/pydar2/smart/" + self.__config.getBuildMachineId() + "/" + self.getBuildRootTag() + ".conf"
        if os.path.exists(self.yumConfFile):
            self.__cat.debug("using userdefined yum conf: " + self.yumConfFile)
        else:
            self.yumConfFile = "/etc/pydar2/yum/" + self.getBuildRootTag() + ".conf"
            self.__cat.debug("using normal yum conf: " + self.yumConfFile)
        self.yumCacheDir = "/var/lib/pydar2/yum/cache/" + self.getBuildRootTag()
        self.rootdir = "/var/lib/pydar2/roots/yum-" + self.getBuildRootTag()
        
        # this definetely has to move to a config file
        self.minimalRpmsPart1 = "basesystem filesystem setup bash glibc"
        self.minimalRpmsPart2 = "coreutils findutils"
        self.minimalRpmsPart3 = "rpm-devel rpm-build make gcc tar gzip patch unzip bzip2 diffutils cpio elfutils redhat-rpm-config perl"
        self.minimalRpmsPart4 = ""
        brr.logDebugLine("rootdir:" + self.rootdir)
    
    def installRpms(self,builddepsarr):
        brr = BuildRootResult(self)
        # todo auto remove all deps which are already installed , not only perl
        if len(builddepsarr) > 0 and not (len(builddepsarr) == 1 and builddepsarr[0] ==  "perl"):
            commandpart = ""
            for b in builddepsarr:
                commandpart = commandpart + " " + b
                cacheParam = ""
                if self.__useCacheOnly:
                    cacheParam = " -C "
            cmd = "sudo yum " + cacheParam + " -c " + self.yumConfFile + " -y -t --installroot=" + self.rootdir + " install " + commandpart
            self.logCommand(cmd,brr)
        return brr
        
    # makes a new root
    def prepareRoot(self):
        brr = BuildRootResult(self)
        cmd = []
        # create some directories first
        cmd.append("sudo mkdir -p " + self.rootdir + "/var/lib/rpm " + self.rootdir + "/var/log " + self.rootdir + "/var/cache " + self.rootdir + "/etc " + self.rootdir + "/proc " + self.rootdir + "/sys " + self.rootdir + "/usr/src/redhat/BUILD " + self.rootdir + "/usr/src/redhat/RPMS " + self.rootdir + "/usr/src/redhat/SOURCES " + self.rootdir + "/usr/src/redhat/SPECS " + self.rootdir + "/usr/src/redhat/SRPMS " + self.rootdir + "/dev/")
        # copy fstab
        cmd.append("sudo cp /etc/fstab " + self.rootdir + "/etc")
        # mount proc
        cmd.append("sudo mount -t proc none " + self.rootdir + "/proc")
        # mount sys
        cmd.append("sudo mount -t sysfs none " + self.rootdir + "/sys")
        # create /dev/null
        cmd.append("sudo mknod " + self.rootdir + "/dev/null c 1 3")
        # rpm initdb
        cmd.append("sudo rpm --root=" + self.rootdir + " --initdb")
        # use a central yum cache
        cmd.append("sudo ln -s " + self.yumCacheDir + " " + self.rootdir + "/var/cache/yum")
        # install the rpms
        cacheParam = ""
        if self.__useCacheOnly:
            cacheParam = " -C "
        if self.minimalRpmsPart1 != "":
            cmd.append("sudo yum " + cacheParam  + " -c " + self.yumConfFile + " -y -t --installroot=" + self.rootdir + " install " + self.minimalRpmsPart1)
        if self.minimalRpmsPart2 != "":
            cmd.append("sudo yum " + cacheParam  + " -c " + self.yumConfFile + " -y -t --installroot=" + self.rootdir + " install " + self.minimalRpmsPart2)
        if self.minimalRpmsPart3 != "":
            cmd.append("sudo yum " + cacheParam  + " -c " + self.yumConfFile + " -y -t --installroot=" + self.rootdir + " install " + self.minimalRpmsPart3)
        if self.minimalRpmsPart4 != "":
            cmd.append("sudo yum " + cacheParam  + " -c " + self.yumConfFile + " -y -t --installroot=" + self.rootdir + " install " + self.minimalRpmsPart4)
        cmd.append("sudo chmod -R a+rwx " + self.rootdir + "/usr/src/redhat/")
        self.logCommands(cmd,brr)
        return brr
        
    def logCommands(self,cmdArr,brr):
        for cmd in cmdArr:
            self.logCommand(cmd,brr)
            # todo: if error => stop
        
    def logCommand(self,cmd,brr):
        brr.logDebugLine("CMD: Command line:" + cmd)
        (status,output) = commands.getstatusoutput(cmd)
        brr.logDebugLine("CMD: Status:" + str(status))
        # first remove all the '\rInstalling: .*? [0-9][0-9]? % done [0-9]*\/[0-9]*
        output = re.sub('\rInstalling: .*? [0-9][0-9]? % done [0-9]*\/[0-9]*',"",output)
        output = re.sub('\r',"",output)
        brr.logDebugLine("CMD: output:" + str(output))
        if status != 0:
            brr.logErrorLine("CMD: status (" + str(status) + ") is not 0 !")
        
    def logCommandWithoutErrors(self,cmd,brr):
        brr.logDebugLine("CMD: Command line:" + cmd)
        (status,output) = commands.getstatusoutput(cmd)
        brr.logDebugLine("CMD: Status:" + str(status))
        # first remove all the '\rInstalling: .*? [0-9][0-9]? % done [0-9]*\/[0-9]*
        output = re.sub('\rInstalling: .*? [0-9][0-9]? % done [0-9]*\/[0-9]*',"",output)
        output = re.sub('\r',"",output)
        brr.logDebugLine("CMD: output:" + str(output))
        
    def buildRpm(self,command):
        brr = BuildRootResult(self)
        self.__cat.debug("start")
        # copy the sources
        self.__copySources(command,brr)
        # create a script which does the actual building
        scriptcontents = self.__createScript(command,brr)
        # then we call the script
        self.__callBuildScript(scriptcontents, command, brr)
        # get all the results
        self.__getLogsAndRpmsAndSrpm(command,brr)
        # then we parse the log: get additional usefull files if need, for example config.log if the build has failed
        self.__parseLog(command,brr)
        return brr
    
    def __copySources(self,command,brr):
        self.__cat.debug("start")
        startdir = command.getBuildDir()
        listdir = os.listdir(startdir)
        specdestdir = os.path.join(self.rootdir,'usr/src/redhat/SPECS')
        sourcesdestdir = os.path.join(self.rootdir,'usr/src/redhat/SOURCES')
        self.__cat.debug('specdestdir: ' + specdestdir)
        self.__cat.debug('sourcesdestdir: ' + sourcesdestdir)
        for entry in listdir:
            rindex = string.rfind(entry,'.spec')
            if os.path.isfile(os.path.join(startdir,entry)) and not (rindex > 0 and rindex == len(entry) - len('.spec')):
                # its not the spec files
                shutil.copy(os.path.join(startdir,entry), sourcesdestdir)
            else:
                # its the spec file
                shutil.copy(os.path.join(startdir,entry),specdestdir)
        self.__cat.debug('copy done')
            
    def __getLogsAndRpmsAndSrpm(self,command,brr):
        self.__cat.debug("start")
        shutil.copy(os.path.join(self.rootdir, "buildlog.txt"), command.getBuildDir())
        cmd = "gzip " + command.getBuildDir() + "/buildlog.txt"
        self.logCommand(cmd,brr)
        # search in the /usr/src/redhat/RPMS and /usr/src/redhat/SRPMS dirs..
        rpmsdir = os.path.join(self.rootdir,'usr/src/redhat/RPMS')
        ld = os.listdir(rpmsdir)
        for entry in ld:
            self.__getRpmsOrSrpms(command,brr,os.path.join(rpmsdir,entry))
        srpmsdir = os.path.join(self.rootdir,'usr/src/redhat/SRPMS')
        self.__getRpmsOrSrpms(command,brr,srpmsdir)
            
    def __getRpmsOrSrpms(self,command,brr,path):
        self.__cat.debug('start, path=' + path)
        ld  = os.listdir(path)
        for entry in ld:
            rindex = string.rfind(entry,'.rpm')
            if os.path.isfile(os.path.join(path,entry)) and (rindex > 0 and rindex == len(entry) - len('.rpm')):
                # let's copy it
                shutil.copy(os.path.join(path,entry),command.getBuildDir())
                rindex = string.rfind(entry,'.src.rpm')
                if  rindex > 0 and rindex == len(entry) - len('.src.rpm'):
                    brr.setSrpm(entry)
                else:
                    brr.addRpm(entry)
    
    def __createScript(self,command,brr):
        self.__cat.debug("start")
        script = "#!/bin/bash\n"
        script = script + 'for i in ' + self.rootdir + '/etc/profile.d/*.sh; do' + "\n"
        script = script + '    if [ -r "$i" ]; then' + "\n"
        script = script + '        . $i' + "\n"
        script = script + "    fi\n"
        script = script + "done\n"
        script = script + "chown -R root.root " + self.rootdir + "/usr/src/redhat\n"
        # the following command first checks if the dependencies are ok
        # ex: --define '_topdir /var/lib/pydar2/roots/yum-fc3-i386/usr/src/redhat'
        script = script + "(/usr/bin/rpmbuild --define '_topdir " + self.rootdir + "/usr/src/redhat' " + command.getDefines() + " --root=" + self.rootdir + " --nobuild --bs " + self.rootdir + "/usr/src/redhat/SPECS/" + command.getSpecFileShortFileName() + " && echo \"deps check ok\" && "
        # do the real build
        script = script + "/usr/sbin/chroot " + self.rootdir + " /usr/bin/rpmbuild --nodeps -ba /usr/src/redhat/SPECS/" + command.getSpecFileShortFileName() + " " + command.getDefines() + " ) > " + self.rootdir + "/buildlog.txt 2>&1"
        self.__cat.debug( "script contains: " + script)
        return script
        
    def __callBuildScript(self, scriptcontents, command, brr):
        self.__cat.debug("start")
        # first write it to a temp file
        filepath = self.rootdir + "/tmp/pydar2build.sh"
        fd = open(filepath,"w")
        fd.write(scriptcontents)
        fd.close()
        cmd = "sudo /bin/bash " + self.rootdir + "/tmp/pydar2build.sh"
        self.logCommand(cmd,brr)
    
    def __parseLog(self,command,brr):
        self.__cat.debug("start, not ready")
    
## tests
#myRoot = YumBasedBuildRoot("","fedora-3-i386-core")
#myRoot.cleanRoot()
#myRoot.prepareRoot()
#spec = SpecFile('/home/dries/projects/rap-svn/svn/trunk/rpms/aldo/aldo.spec',' --define "fc3 1" ')
#myRoot.installRpms(spec.getBuildRequires())
##myRoot.
#print myRoot.logText
