
""" This class represents one target: for example rpmforge-i386-stable """

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# Copyright 2004 Dries Verachtert

import config
import storagefactory
from mastercommand import MasterCommand
from log4py import Logger
from preparespecfilescriptlist import PrepareSpecFileScriptList
import posixpath, posix, string, os, sha
from buildrootresult import BuildRootResult
import smtplib
from email.MIMEText import MIMEText
from email.MIMEMultipart import MIMEMultipart
from email.MIMEBase import MIMEBase
from email import Encoders

class Target:
    def __init__(self, name):
        self.__cat = Logger().get_instance(self)
        self.__name = name
        self.__id = -1
        self.__prepareSpecFileScriptList = PrepareSpecFileScriptList()
        self.__acceptCommandScript = None
        self.__rpmsDir = None
        self.__getRpmDataScript = None
        self.__moveCommandResultsScript = None
        
    def setRpmsDir(self, rpmsDir):
        self.__rpmsDir = rpmsDir
        
    def getRpmsDir(self):
        return self.__rpmsDir
    
    def getName(self):
        return self.__name

    def supportsAutoQueueing(self):
        retval = False
        self.__cat.debug("__rpmsDir:" + str(self.__rpmsDir))
        self.__cat.debug("__acceptCommandScript:" + str(self.__acceptCommandScript))
        self.__cat.debug("__moveCommandResultsScript:" + str(self.__moveCommandResultsScript))
        
        if self.__rpmsDir == None or self.__acceptCommandScript == None or self.__getRpmDataScript == None or self.__moveCommandResultsScript == None:
            retval = False
        else:
            retval = True
        self.__cat.debug("retval: " + str(retval))
        return retval
        
    def getId(self):
        return self.__id
        
    def setId(self,id):
        self.__id = id

    def appendPrepareSpecFileScript(self,newScript):
        self.__prepareSpecFileScriptList.appendScript(newScript)
        
    def runPrepareSpecFileScripts(self,aCommand,commanddir, brr):
        self.__prepareSpecFileScriptList.runScripts(aCommand,commanddir, brr)

    def setGetRpmDataScript(self, newScript):
        self.__getRpmDataScript = newScript
        
    def setAcceptCommandScript(self,newScript):
        self.__acceptCommandScript = newScript
        
    def setMoveCommandResultsScript(self,newScript):
        self.__moveCommandResultsScript = newScript

    def updateTargetFileList(self, userId, mailTo):
        self.__cat.debug("start")
        self.__storage = storagefactory.StorageFactory.getStorage()
        if not self.supportsAutoQueueing():
            return
        self.__updateListOfRpms()
        self.__cat.debug("updateListOfRpms is done")
        
        
    def autoQueue(self, mailTo, userName, priority, specRepository, distroArchTag):
        self.__cat.debug("start")
        self.__storage = storagefactory.StorageFactory.getStorage()
        if not self.supportsAutoQueueing():
            return
        self.__cat.debug("updateListOfRpms not done")
        self.__autoQueueByDistroArchTag(distroArchTag,mailTo,userName,priority,specRepository)
        
    def autoQueueOld(self, mailTo, userName, priority, specRepository):
        self.__cat.debug("start")
        self.__storage = storagefactory.StorageFactory.getStorage()
        if not self.supportsAutoQueueing():
            return
        for datag in self.__storage.getDistroArchTags():
            self.__cat.debug("going to autoqueue for distroarchtag:" + datag)
            self.__autoQueueByDistroArchTag(datag, mailTo, userName, priority, specRepository)
            
    def __autoQueueByDistroArchTag(self,datag, mailTo, userName, priority, specRepository):
        self.__cat.debug("start, datag=" + datag)
        self.__reloadAcceptCommandScript()
        queuecounter = 0
        brr = BuildRootResult(None)
        brr.logDebugLine("autoQueue log for DistroArchTag:" + str(datag) + ",mailTo:" + str(mailTo) + ",userName:"+ str(userName) + ",priority:" + str(priority) + ",specRepository.name:" + specRepository.getName() + ",target.name:"+ self.getName())
        # create a hash which contains:   specfileid -> version   for each specfile in the specrepository
        # create a hash which contains:   a specfileid -> version for each rpm in the list of rpms of this target, when a specfileid can be found
        
        specRepoSpecFileIdsToVersions = self.__storage.getSpecFileIdsToVersions(specRepository)
        targetRpmsSpecFileIdsToVersions = self.__storage.getTargetRpmsSpecFileIdsToVersions(self, datag,specRepository)
        k = specRepoSpecFileIdsToVersions.keys()
        k.sort(lambda b, c: - cmp( specRepoSpecFileIdsToVersions[b], specRepoSpecFileIdsToVersions[c]))
        for spid in k:
            if queuecounter < config.Config.getInstance().getMaxAutoQueueCount():
                if (spid not in targetRpmsSpecFileIdsToVersions.keys()) or (specRepoSpecFileIdsToVersions[spid] > targetRpmsSpecFileIdsToVersions[spid]):
                    if self.autoQueueOne(spid, datag, mailTo, userName, priority, specRepository, brr):
                        queuecounter = queuecounter + 1
                else:
                    self.__cat.debug("not queuing: specFileId=" + str(spid) + ",distroArchTag=" + datag + ",mailto=" + mailTo + ",userName=" + userName + ",priority=" +str(priority) +",specRepository=" + str(specRepository))
                    targetVersion = "NONE"
                    if spid in targetRpmsSpecFileIdsToVersions.keys():
                        targetVersion = targetRpmsSpecFileIdsToVersions[spid]
                    specObj = specRepository.getSpecRepositorySpecFileById(spid)
                    pathFromRoot = "NONE"
                    if specObj != None:
                        pathFromRoot = specObj.getPathFromRoot()
                    brr.logDebugLine("specRepoSpecId " + str(spid) + " not queued: " + "specRepo version: " + str(specRepoSpecFileIdsToVersions[spid]) + ",targetRpm Version:" + str(targetVersion) + ",path:" + str(pathFromRoot))
        self.__informClientAfterAutoQueueByDistroArchTag(brr, mailTo, datag)
                
    def __informClientAfterAutoQueueByDistroArchTag(self, brr, mailTo, datag):
        self.__cat.debug("start")
        subject = "pydar2:: autoqueue: " + str(datag)
        self.__cat.debug("subject set")
        output = brr.getContainsErrorsText() + "\n"
        self.__cat.debug("errors descr added")
        output = output + "\nLog:\n"
        output = output + brr.getLog() + "\n"
        msg = MIMEMultipart()
        msg['Subject'] = subject
        msg['From'] = "pydar-dries@vmhosting.org"
        msg['To'] = mailTo
        me = "pydar-dries@vmhosting.org"
        you = mailTo
        msg.attach(MIMEText(output))
        s = smtplib.SMTP("127.0.0.1")
        s.sendmail(me, [you], msg.as_string())
        s.close()

        
    def autoQueueOne(self, specFileId, distroArchTag, mailTo, userName, priority, specRepository, brr):
        self.__cat.debug("start, specFileId=" + str(specFileId) + ",distroArchTag=" + distroArchTag + ",mailto=" + mailTo + ",userName=" + userName + ",priority=" +str(priority) +",specRepository=" + str(specRepository))
        testCommand = MasterCommand("BUILDSPECBYPATH",userName,specRepository.getId(),specFileId,mailTo,distroArchTag,priority,self.getId())
        specObj = specRepository.getSpecRepositorySpecFileById(specFileId)
        pathFromRoot = "NONE"
        if specObj != None:
            pathFromRoot = specObj.getPathFromRoot()
        if (specObj != None) and self.acceptCommand(testCommand):
            config.Config.getInstance().getCommandList().addCommand("BUILDSPECBYPATH",userName,specRepository.getName(),pathFromRoot,mailTo,distroArchTag,priority,self.getName())
            brr.logDebugLine("accepted: id: " + str(specFileId) + ", path: " + pathFromRoot)
            return True
        else:
            brr.logDebugLine("not accepted: id: " + str(specFileId) + ", path: " + pathFromRoot)
            return False
        
        
    # get some data from an srpm
    def getRpmData(self, srpmPath):
        if self.__getRpmDataScript != None:
            return self.__getRpmDataScript.runScript(srpmPath)
        else:
            return None
    
    def __getCheckSum(self,fileName):
        #self.__cat.debug("fileName: " + fileName)
        fullFileName = os.path.join(self.getRpmsDir(), fileName)
        type = config.Config.getInstance().getCheckSumType()
        if type == "SHA":
            s = sha.new()
            f = open(fullFileName, 'r')
            line = f.readline()
            while line != '':
                s.update(line)
                line = f.readline()
            f.close()
            return s.hexdigest()
        else:
            # file modification date + size
            return str(os.path.getmtime(fullFileName)) + "-" + str(os.path.getsize(fullFileName))
    
    # this updates the list of rpms in the database
    # with this list and the list of specrepositoryspecfiles, it is possible to create a todo list
    def __updateListOfRpms(self):
        # recursively check all the rpms
        topDir = self.getRpmsDir()
        arr = self.__recursiveFindFiles(topDir, "")
        myFileNameToChecksum = self.__storage.getTargetFileListToChecksums(self)
        for fileName in arr:
            try:
                checksum = self.__getCheckSum(fileName)
                if fileName in myFileNameToChecksum.keys():
                    if checksum != myFileNameToChecksum[fileName]:
                        # rpm has changed?
                        self.__cat.debug("changed rpm")
                        (name, version, distroarchtag) = self.getRpmData(os.path.join(self.getRpmsDir(),fileName))
                        self.__cat.debug("going to update in db")
                        self.__storage.updateTargetFile(self, fileName, checksum, version, distroarchtag, name)
                else:
                    self.__cat.debug("new rpm")
                    (name, version, distroarchtag) = self.getRpmData(os.path.join(self.getRpmsDir(),fileName))
                    self.__cat.debug("going to insert in db")
                    self.__storage.insertTargetFile(self, fileName, checksum, version, distroarchtag, name)
            except:
                self.__cat.debug("problem with rpm: " + str(fileName))
        # todo remove the data from the database about files which dont exist anymore    
        
    def __recursiveFindFiles(self,topDir,startDir):
        #self.cat.debug("start, startDir=" + startDir)
        arr = []
        fullDir = posixpath.join(topDir,startDir)
        topDirList = posix.listdir(fullDir)
        for entryInDir in topDirList:
            #self.cat.debug("entryInDir=" + entryInDir)
            fullEntryInDir = posixpath.join(fullDir,entryInDir)
            if posixpath.isdir(fullEntryInDir) and not os.path.islink(fullEntryInDir):
                #self.cat.debug("it's a dir!")
                # recurse
                tmpArr = self.__recursiveFindFiles(topDir,posixpath.join(startDir,entryInDir))
                arr = arr + tmpArr
            else:
                # it's not a dir
                rindex = string.rfind(entryInDir,'.rpm')
                rindex2 = string.rfind(entryInDir, '.src.rpm')
                rindex3 = string.rfind(entryInDir, '-debuginfo-')
                if rindex > 0 and rindex == len(entryInDir) - len('.rpm') and rindex2 < 0 and rindex3 < 0:
                    arr.append(os.path.join(startDir,entryInDir))
        return arr
           
    # check if a certain command is accepted for this target or not
    def acceptCommand(self, command):
        if self.__acceptCommandScript != None:
            return self.__acceptCommandScript.acceptCommand(command)
        else:
            return True
    
    def __reloadAcceptCommandScript(self):
        if self.__acceptCommandScript != None:
            self.__acceptCommandScript.reloadMethod()

    # move the results of a command (an srpm, rpm(s), buildlog.txt.gz, .. ) to their final place
    def moveCommandResults(self, command, commanddir):
        if self.__moveCommandResultsScript != None:
            return self.__moveCommandResultsScript.runScript(command, commanddir)
        else:
            return False
